/* ======================================================================================================================================
  @author Grant McKenzie (grant.mckenzie@geog.ucsb.edu)
  @date 02/02/2014
  @project Project 2 
  @course MAT259
  @description This project shows the individual Dewey Classes for the books in the Seattle Public Library styled by check-out signature
  @file utilities file doing all the heavy lifting
  ======================================================================================================================================== */
  
// @input 2D array of sorted dewey classes and 2D array of text dewey values
ArrayList<String[]> getColorDefinition(float[][] sorted, String[][] dewey) {
      String superclass = "";
      ArrayList<String[]> colorBlocks = new ArrayList<String[]>();
      int count = 0;
      for(int i=0;i<sorted.length;i++) {
        float t = 0;
        float morning = 0;
        float noon = 0;
        float even = 0;
        // Divide morning, noon and night by the hour of the check-out
        for (int j=8; j < 20; j++){
            // println(sorted[1][j]);
            if (j <= 11)
              morning += sorted[i][j];
            else if (j > 11 && j <= 15)
              noon += sorted[i][j];
            else if (j > 15)
              even += sorted[i][j];
            t += sorted[i][j];
        }
        // Make sure that the Dewey class actually has had some activity
        if (t > 0.0) {
          
          if (round(int(dewey[i][0])/10) == 0)
             superclass = "000";
          else if (round(int(dewey[i][0])/10) == 0)
             superclass = "010";
          else
             superclass = dewey[i][0].substring(0,1) + "00";
             
         // Create an array of all these wonderful Dewey Properties
         String[] a = new String[6];
         a[0] = superclass;
         a[1] = dewey[i][1];
         a[2] = dewey[i][0];
         a[3] = (round(morning*255))+"";
         a[4] = (round(noon*255))+"";
         a[5] = (round(even*255))+"";
         colorBlocks.add(a);
       
         count++;
        }
      }
      return colorBlocks;
}

// Read the CSV in to a 2D array given the filename and number of dimensions (columns)

String[][] loadCSV(String filename, int dimensions) {
  String lines[] = loadStrings(filename);
  String[][] d = new String [lines.length][dimensions];
  
  //parse values into 2d array
  for (int i=0; i < lines.length; i++) {
    String [] temp = new String [lines.length];
    // println(lines);
    temp= split(lines[i], ',');
    // println(temp[1] + "\t" +temp.length);
    for (int j=0; j < temp.length; j++){
     d[i][j]=temp[j];
    }
  }
  return d;
}

// Sort the Dewey system to get it in the correct format for styling
// This is definitely not the most efficient way, but it works and is negligible for such a small dataset.
float[][] SortDewey(String[][]dew) {
   float[][] AggDewClass = new float[dew.length][24];
   float[] TotDewClass = new float[dew.length];
   for(int i=0;i<dew.length;i++) {
     AggDewClass[int(dew[i][0])][int(dew[i][1])] = int(dew[i][2]);
     TotDewClass[int(dew[i][0])] += int(dew[i][2]);
   }
   for(int i=0;i<dew.length;i++) {
     AggDewClass[int(dew[i][0])][int(dew[i][1])] = int(dew[i][2]) / TotDewClass[int(dew[i][0])];
   }
   return AggDewClass;
}

// Set a color gradient for the legend
void setGradient(int x, int y, float w, float h, color c1, color c2, int axis ) {
  for (int i = x; i <= x+w; i++) {
    float inter = map(i, x, x+w, 0, 1);
    color c = lerpColor(c1, c2, inter);
    stroke(c);
    line(i, y, i, y+h);
  }
}
